@@ -10,7 +10,7 @@ from queue import Empty as QueueEmpty
 from argparse_addons import Integer
 
 from ..client import Client
-from ..common import hexlify
+from . import format_message
 
 
 class QuitError(Exception):
@@ -29,7 +29,18 @@ class ClientThread(threading.Thread):
         self._client_id = args.client_id
         self._keep_alive_s = args.keep_alive
         self._session_expiry_interval = args.session_expiry_interval
-        self._topics = args.subscribe
+        self._subscriptions = []
+
+        retain_handling = args.retain_handling[:len(args.subscribe)]
+
+        if not retain_handling:
+            retain_handling = [0]
+
+        remaining = len(args.subscribe) - len(retain_handling)
+        retain_handling += remaining * [retain_handling[-1]]
+
+        for topic, retain_handling in zip(args.subscribe, retain_handling):
+            self._subscriptions.append((topic, retain_handling))
 
     async def main(self):
         if self._cafile:
@@ -46,7 +57,7 @@ class ClientThread(threading.Thread):
                         self._client_id,
                         keep_alive_s=self._keep_alive_s,
                         session_expiry_interval=self._session_expiry_interval,
-                        subscriptions=self._topics,
+                        subscriptions=self._subscriptions,
                         topic_alias_maximum=10,
                         ssl=context)
 
@@ -72,8 +83,10 @@ class Monitor(object):
     def __init__(self, stdscr, args):
         self._stdscr = stdscr
         self._sorted_topics = []
+        self._messages = {}
         self._formatted_messages = {}
         self._playing = True
+        self._format = 'auto'
         self._modified = True
         self._queue = Queue()
         self._nrows, self._ncols = stdscr.getmaxyx()
@@ -135,7 +148,7 @@ class Monitor(object):
     def draw_menu(self, row):
         self.addstr_color(row,
                           0,
-                          self.stretch('q: Quit, p: Play/Pause'),
+                          self.stretch(f'q: Quit, p: Play/Pause, f: Format ({self._format})'),
                           curses.color_pair(2))
 
     def addstr(self, row, col, text):
@@ -166,16 +179,23 @@ class Monitor(object):
             raise QuitError()
         elif key == 'p':
             self._playing = not self._playing
+        elif key == 'f':
+            self._format = {
+                'auto': 'binary',
+                'binary': 'text',
+                'text': 'auto'
+            }[self._format]
 
-    def try_update_message(self):
-        timestamp, topic, message = self._queue.get_nowait()
+            for topic, (timestamp, message) in self._messages.items():
+                self.format_message(timestamp, topic, message)
 
-        if topic is None:
-            sys.exit('Broker connection lost!')
+            self._modified = True
 
+    def format_message(self, timestamp, topic, message):
         lines = []
         row_length = max(1, self._ncols - 12)
-        message = hexlify(message)
+        message = format_message(self._format, message)
+        message = message.replace('\x00', '\\x00')
 
         for i in range(0, len(message), row_length):
             lines.append(message[i:i + row_length])
@@ -184,6 +204,15 @@ class Monitor(object):
         formatted += [11 * ' ' + line for line in lines]
         self._formatted_messages[topic] = formatted
 
+    def try_update_message(self):
+        timestamp, topic, message = self._queue.get_nowait()
+
+        if topic is None:
+            sys.exit('Broker connection lost!')
+
+        self.format_message(timestamp, topic, message)
+        self._messages[topic] = (timestamp, message)
+
         if topic not in self._sorted_topics:
             self.insort(topic)
 
@@ -260,8 +289,16 @@ def add_subparser(subparsers):
         '--no-check-hostname',
         action='store_true',
         help='Do not check certificate hostname.')
+    subparser.add_argument(
+        '--retain-handling',
+        type=Integer(0, 2),
+        default=[],
+        action='append',
+        help=('Retain handling for the subscriptions. May be given once for each '
+              'subscription. Last known value is used for remaining topics '
+              '(default: 0).'))
     subparser.add_argument(
         'subscribe',
-        nargs='*',
+        nargs='+',
         help='Subscribe to given topic(s) <topic>.')
     subparser.set_defaults(func=_do_monitor)
